//
//  CitiesTableViewController.swift
//  El Vuelo del Grajo
//
//  Created by Santiago Pavón on 7/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit

class CitiesTableViewController: UITableViewController {

    // Fichero donde salvo los nombres de las ciudades.
    fileprivate let SAVED_FILENAME = "cities.plist"
    
    // Las ciudades a mostrar en la tabla.
    var cities: [String]!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        self.navigationItem.leftBarButtonItem = self.editButtonItem
        
        loadCities()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return cities.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "City Cell", for: indexPath) 

        cell.textLabel?.text = cities[indexPath.row]
        
        return cell
    }

    
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return "Selecciona una Ciudad"
    }
    
    
    override func tableView(_ tableView: UITableView, titleForFooterInSection section: Int) -> String? {
        return "IWEB 2016/2017"
    }
    
    
    /*
    // Override to support conditional editing of the table view.
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        // Return NO if you do not want the specified item to be editable.
        return true
    }
    */

    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {

            cities.remove(at: indexPath.row)
            
            tableView.deleteRows(at: [indexPath], with: .fade)
            
            saveCities()
            
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }    
    }

    // Override to support rearranging the table view.
    override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to toIndexPath: IndexPath) {

        let aux = cities[fromIndexPath.row]
        
        cities.remove(at: fromIndexPath.row)
        cities.insert(aux, at: toIndexPath.row)
        
        saveCities()
    }

    // Override to support conditional rearranging of the table view.
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
        return true
    }

    
    // MARK: Table View Delegate
    
    override func tableView(_ tableView: UITableView, titleForDeleteConfirmationButtonForRowAt indexPath: IndexPath) -> String? {
        return "ñam ñam"
    }
    
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {

        if segue.identifier == "Show City" {
            
            if let cwvc = segue.destination as? CityWeatherViewController {
                
                if let row = tableView.indexPath(for: sender as! UITableViewCell)?.row {
                
                    cwvc.city = cities[row]
                }
            }
            
        } else if segue.identifier == "Add City" {
            
            cities.insert("", at: 0)
            
            let ip = IndexPath(row: 0, section: 0)
            
            tableView.insertRows(at: [ip], with: UITableViewRowAnimation.right)
            
            tableView.selectRow(at: ip, animated: true, scrollPosition: .top)
            
            /*
            tableView.scrollToRowAtIndexPath(ip, atScrollPosition: UITableViewScrollPosition.Top, animated: true)
            */

            if let cevc = segue.destination as? CityEditorViewController {
                
                cevc.city = ""
            }
            
        } else if segue.identifier == "Edit City" {
            
            if let cevc = segue.destination as? CityEditorViewController {

                if let ip = tableView.indexPath(for: sender as! UITableViewCell) {

                    tableView.selectRow(at: ip, animated: true, scrollPosition: .top)
                    
                    cevc.city = cities[ip.row]
                }
            }
        }
    }

    
    @IBAction func cityWasEdited(_ segue: UIStoryboardSegue) {
                
        if let cevc = segue.source as? CityEditorViewController {

            if let ip = tableView.indexPathForSelectedRow {
                
                cities[ip.row] = cevc.city
                
                tableView.reloadRows(at: [ip], with: .automatic)
                
                saveCities()
            }
            
        }
    }
    
    // MARK: Persistencia
    
    fileprivate func getSavedCitiesPath() -> String {
        let path = NSSearchPathForDirectoriesInDomains(.documentDirectory,
            .userDomainMask, true)[0] 
        
        return (path as NSString).appendingPathComponent(SAVED_FILENAME)
    }
    
    
    fileprivate func loadCities() {
        let path = getSavedCitiesPath()
        if let cities = NSArray(contentsOfFile: path) as? [String] {
            self.cities = cities
        } else {
            cities = ["Madrid","Paris","Lisboa","Londres","Moscu",
                "Caracas","Tripoli","Tokio"]
        }
    }
    
    fileprivate func saveCities() {
        let path = getSavedCitiesPath()
        (cities as NSArray).write(toFile: path, atomically: true)
    }
}
